define(['events', 'appSettings', 'playbackManager'], function (events, appSettings, playbackManager) {
    "use strict";

    function getSubtitleInfos(options) {

        var enabled = false;
        var tracks = [];
        var trackNames = [];

        var mediaSource = options.mediaSource;
        var item = options.item;

        var currentTrack;

        if (mediaSource.DefaultSubtitleStreamIndex != null) {
            var mediaStreams = options.mediaSource.MediaStreams || [];
            for (var i = 0, length = mediaStreams.length; i < length; i++) {
                if (mediaStreams[i].Type === 'Subtitle' && mediaStreams[i].Index === mediaSource.DefaultSubtitleStreamIndex) {
                    currentTrack = mediaStreams[i];
                    enabled = true;
                    break;
                }
            }
        }

        if (currentTrack && item) {
            if (currentTrack.DeliveryMethod === 'External') {

                var subUrl = playbackManager.getSubtitleUrl(currentTrack, item.ServerId);
                if (subUrl) {
                    tracks.push(subUrl);
                    trackNames.push(currentTrack.DisplayTitle);
                }
            }
        }

        return {

            enabled: enabled,
            tracks: tracks,
            trackNames: trackNames
        };
    }

    return function () {

        var self = this;

        self.name = 'External Player';
        self.type = 'mediaplayer';
        self.id = 'externalplayer';

        // Prioritize first
        self.priority = -1;
        self.isLocalPlayer = true;
        self.isExternalPlayer = true;

        var currentSrc;

        self.canPlayMediaType = function (mediaType) {

            if (mediaType === 'Video') {

                return appSettings.enableSystemExternalPlayers();
            }

            return false;
        };

        self.canPlayItem = function (item, playOptions) {

            if (!playOptions.fullscreen) {
                return false;
            }

            return true;
        };

        self.currentSrc = function () {
            return currentSrc;
        };

        self.play = function (options) {

            var streamUrl = options.url;

            // just lie and say mp4 to maximize the number of players that will be presented

            var title = options.item ? options.item.Name : null;

            var positionMs = parseInt((options.playerStartPositionTicks || 0) / 10000);
            var subtitleUrl = null;
            var subtitleName = null;
            var subtitlesEnabled = false;

            var subtitleInfos = getSubtitleInfos(options);

            if (subtitleInfos.tracks.length) {
                subtitleUrl = subtitleInfos.tracks[0];
                subtitleName = subtitleInfos.trackNames[0];
                subtitlesEnabled = subtitleInfos.enabled;
            }

            if (MainActivity.launchMediaIntent(streamUrl, title, 'video/mp4', positionMs, subtitleUrl, subtitleName, subtitlesEnabled)) {
                self._currentTime = positionMs;
                return Promise.resolve();
            } else {
                // TODO: Display some message why not external player could be started
                // like: No Activity found to handle Intent { act=android.intent.action.VIEW dat=/storage/emulated/0/Android/data/com.mb.android/files/sync/,,,,,
                return Promise.reject();
            }
        };

        self.setSubtitleStreamIndex = function (index) {
        };

        self.canSetAudioStreamIndex = function () {
            return false;
        };

        self.setAudioStreamIndex = function (index) {

        };

        // Save this for when playback stops, because querying the time at that point might return 0
        self.currentTime = function (val) {
            return this._currentTime;
        };

        self.duration = function (val) {
            return null;
        };

        self.stop = function (destroyPlayer, reportEnded) {

            return closePlayer().then(function () {
                onEndedInternal(reportEnded);
                return Promise.resolve();
            });
        };

        self.destroy = function () {
            closePlayer();
        };

        function closePlayer() {

            return Promise.resolve();
        }

        self.pause = function () {
        };

        self.unpause = function () {
        };

        self.paused = function () {
            return false;
        };

        self.volume = function (val) {
        };

        self.setMute = function (mute) {
        };

        self.isMuted = function () {
        };

        function onEnded() {

            onEndedInternal(true);
        }

        function onError() {

            events.trigger(self, 'error');
        }

        function onEndedInternal(triggerEnded) {

            if (triggerEnded) {
                var stopInfo = {
                    src: currentSrc
                };

                events.trigger(self, 'stopped', [stopInfo]);
            }

            currentSrc = null;
        }

        self.getDeviceProfile = function (item) {
            var profile = {};

            // Only setting MaxStaticBitrate for older servers
            profile.MaxStreamingBitrate = profile.MaxStaticBitrate = 200000000;
            profile.MusicStreamingTranscodingBitrate = 192000;

            profile.DirectPlayProfiles = [];

            // leave container null for all
            profile.DirectPlayProfiles.push({
                Type: 'Video'
            });

            // leave container null for all
            profile.DirectPlayProfiles.push({
                Type: 'Audio'
            });

            profile.TranscodingProfiles = [];

            profile.TranscodingProfiles.push({
                Container: 'mkv',
                Type: 'Video',
                AudioCodec: 'aac,mp3,ac3',
                VideoCodec: 'h264',
                Context: 'Streaming'
            });

            profile.TranscodingProfiles.push({
                Container: 'mp3',
                Type: 'Audio',
                AudioCodec: 'mp3',
                Context: 'Streaming',
                Protocol: 'http'
            });

            profile.ContainerProfiles = [];
            profile.CodecProfiles = [];

            // Subtitle profiles
            // External vtt or burn in
            profile.SubtitleProfiles = [];
            profile.SubtitleProfiles.push({
                Format: 'srt',
                Method: 'External'
            });
            profile.SubtitleProfiles.push({
                Format: 'vtt',
                Method: 'External'
            });
            profile.SubtitleProfiles.push({
                Format: 'ass',
                Method: 'External'
            });
            profile.SubtitleProfiles.push({
                Format: 'ssa',
                Method: 'External'
            });

            profile.SubtitleProfiles.push({
                Format: 'srt',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'subrip',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'ass',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'ssa',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'dvb_teletext',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'dvb_subtitle',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'dvbsub',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'pgs',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'pgssub',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'dvdsub',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'vtt',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'sub',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'idx',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'smi',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'mov_text',
                Method: 'Embed'
            });
            profile.SubtitleProfiles.push({
                Format: 'eia_608',
                Method: 'VideoSideData'
            });
            profile.SubtitleProfiles.push({
                Format: 'eia_708',
                Method: 'VideoSideData'
            });

            profile.ResponseProfiles = [];

            return Promise.resolve(profile);
        };

        window.onExternalPlayerClosed = function (positionMs) {
            self._currentTime = positionMs;
            onEnded();
        };
    };
});